package com.example.model;

import javafx.geometry.Dimension2D;
import javafx.geometry.Point2D;

public class Paddle {

    private final double rightBound;

    private final Dimension2D dimension;

    private Point2D centerPoint;

    public Paddle(double rightBound, Dimension2D dimension, Point2D centerPoint) {
        this.rightBound = rightBound;
        this.dimension = dimension;
        this.centerPoint = centerPoint;
    }


    public void movePaddle(double increment) {
        centerPoint = centerPoint.add(increment, 0);
        limitMovement();
    }

    public void setPaddle(double x) {
        centerPoint = new Point2D(x, centerPoint.getY());
        limitMovement();
    }

    private void limitMovement() {
        double width = dimension.getWidth();
        if (centerPoint.getX() - width / 2 < 0)  {
            centerPoint = new Point2D(width / 2, centerPoint.getY());
        }
        if (centerPoint.getX() + width / 2 > rightBound)  {
            centerPoint = new Point2D(rightBound - width / 2, centerPoint.getY());
        }
    }

    public double getWidth() {
        return dimension.getWidth();
    }

    public boolean intersects(Ball ball) {
        double paddleLX = centerPoint.getX() - dimension.getWidth() / 2;
        double paddleRX = centerPoint.getX() + dimension.getWidth() / 2;
        double paddleRY = centerPoint.getY() + dimension.getHeight() / 2;

        return lineXIntersect(ball, paddleLX, paddleRX, paddleRY);
    }

    private boolean lineXIntersect(Ball ball, double lx, double rx, double y) {
        double ballX = ball.getCenterPoint().getX();
        double ballY = ball.getCenterPoint().getY();
        double ballR2 = Math.pow(ball.getRadius(), 2);

        for (double x = lx; x < rx; x++) {
            if ( ballR2 > Math.pow(ballX - x, 2) + Math.pow(ballY - y, 2)) {
                return true;
            }
        }

        return false;
    }

    public double getTranslateX() {
        return centerPoint.getX() - dimension.getWidth() / 2;

    }

    public double getTranslateY() {
        return centerPoint.getY() - dimension.getHeight() / 2;
    }
}
